#include "sprite.h"

#include "SDL_image.h"
#include <assert.h>
#include <math.h>

#include "locate_data.h"

SDL_Surface *Sprite::screen;

Sprite::Sprite( Uint16 x, Uint16 y, char *file_sprite, char *file_sprite_hover, char *file_sprite_click, const char *file_sound )
{
    sprite = IMG_Load( locate_data(file_sprite) );
    assert( sprite != NULL );
    sprite_hover = file_sprite_hover==NULL? NULL : IMG_Load( locate_data(file_sprite_hover) );
    sprite_click = file_sprite_click==NULL? NULL : IMG_Load( locate_data(file_sprite_click) );
    init( x, y );

    #ifdef AUDIO_SDL
    sound = file_sound==NULL? NULL : Mix_LoadWAV( locate_data(file_sound) );
    #endif
}

void Sprite::init( Uint16 x, Uint16 y )
{
    moveTo( x, y );
    old_rect = rect;

    hover_time = 0;
    run_time = 0;
}

void Sprite::moveTo( Uint16 x, Uint16 y )
{
    dest_x = rect.x = x - sprite->w / 2;
    dest_y = rect.y = y - sprite->h / 2;
    rect.w = sprite->w;
    rect.h = sprite->h;

    state = state_null;
}

void Sprite::slideTo( Uint16 x, Uint16 y )
{
    dest_x = x - sprite->w / 2;
    dest_y = y - sprite->h / 2;
    rect.w = sprite->w;
    rect.h = sprite->h;

    state = state_null;
}

Sprite::~Sprite()
{
    SDL_FreeSurface( sprite );
    if( sprite_hover != NULL ) SDL_FreeSurface( sprite_hover );
    if( sprite_click  != NULL ) SDL_FreeSurface( sprite_click );
    #ifdef AUDIO_SDL
    if( sound != NULL ) Mix_FreeChunk( sound );
    #endif
}

bool Sprite::clear( SDL_Rect *splash )
{
    int old_state = state;
    
    if( dest_x != rect.x || dest_y != rect.y )
    {
	int dx = dest_x - rect.x;
	int dy = dest_y - rect.y;
	float norm = (int)(.5+sqrt( dx*dx + dy*dy ));
	if( norm > 20 )
	{
	    rect.x += int( dx / norm * 20 );
	    rect.y += int( dy / norm * 20 );
	}
	else
	{
	    rect.x = dest_x;
	    rect.y = dest_y;
	}
    }

    if( sprite_click!=NULL && run_time>0 )
	state = state_run;
    else if( sprite_hover!=NULL && hover_time>0 )
	state = state_hover;
    else
	state = state_free;

    if( old_state != state || ( old_rect.x!=rect.x || old_rect.y!=rect.y ) )
    {
	need_redraw = true;
	*splash = old_rect;
	return true;
    }
    else
    {
	return false;
    }
}

bool Sprite::redraw( SDL_Rect *splash )
{
    if( run_time   > 0 ) run_time--;
    if( hover_time > 0 ) hover_time--;

    if( need_redraw )
    {
	need_redraw = false;
	old_rect = rect;
	*splash = rect;
	return true;
    }
    else
    {
	return false;
    }
}

void Sprite::blit()
{
    SDL_Rect dest = rect;
    switch( state )
    {
	case state_run: SDL_BlitSurface( sprite_click, NULL, screen, &dest ); break;
	case state_hover: SDL_BlitSurface( sprite_hover, NULL, screen, &dest ); break;
	case state_free:
	default: SDL_BlitSurface( sprite, NULL, screen, &dest ); break;
    }
}

void Sprite::hover( Uint16 x, Uint16 y )
{
    if( is_hover(x,y) )
    {
	hover_time = 1;
    }
}

bool Sprite::click( Uint16 x, Uint16 y )
{
    return is_hover(x,y);
}

void Sprite::run()
{
    run_time = 10;
    #ifdef AUDIO_SDL
    if( sound != NULL ) Mix_PlayChannel( -1, sound, 0 );
    #endif
}


bool Sprite::is_hover( Uint16 x, Uint16 y )
{
    return x >= rect.x && x < rect.x+rect.w && y >= rect.y && y < rect.y+rect.h;
}
